clear,clc
%-------------------------------------------------------------------------
%GDIFF_cooling.m  is an example of how the main diffusion routine can be
%used to solve the diffusion in garnet at conditions with decreasing T.
%
%This program comes with no warranty and it is intended for educational/
%research purposes. For further details please see documentation at:
%GDIFF: a Finite Difference code for the calculation of multicomponent diffusion in garnet 
%doi: 10.5281/zenodo.7805990
%
%Evangelos Moulas, 16 May 2023, JGU Mainz
%------------------------------------------------------------------------
plot_sim   = 1;       %Plot during simulation
plot_fin   = 1;       %Plot at the end of the simulation
%Parameters of the Physical problem--------------------------------------
Ti         = 973;                 %Initial temperature in K
Pi         = 10000;               %initial Pressure in bar
Myr        = 3600*24*365.25*1e6;  %Seconds in a Myr
dtdiff     = 0.01*Myr;            %Diffusion timestep
CR         =  50/Myr;             %cooling rate in K/Myr
Tstop      =  673;                %temperature where model stops
Rstart     =    0;                %Start of coordinates in microns
Rstop      =  200;                %End of coordinates in microns
%Numerical --------------------------------------------------------------
ndim       = 1;         %Dimension Number
NBC        = 0;         %Neumann boundary condition (left)
nTpath     = 100;       %Resolution of T path.
nrest      = 1;         %Time increments of implicit method
nx         = 100;       %Spatial Discretization
nout       = 100;       %Plot every nout steps (only if plot_sim=1)
%------------------------------------------------------------------------
Tpath      = linspace(Ti,Tstop,nTpath); %Temperature path
tMax       = (Ti-Tstop)./CR;            %calculate max time
tpath      = linspace(0,tMax,nTpath);   %calculate time of path
%Initial Spatial direction ----------------------------------------------
R          = 1e-6*linspace(Rstart,Rstop,nx);  %Total profile
%Initial Concentration
Alm = 0.50*ones(1,nx);  Alm(end) = 0.50;      %Initial profile
Prp = 0.30*ones(1,nx);  Prp(end) = 0.20;      %
Sps = 0.15*ones(1,nx);  Sps(end) = 0.10;      %
Grs = 1-Alm-Prp-Sps;                          %Closure relationship
%Closure-----------------------------------------------------------------
C   = [Alm;Prp;Sps;Grs];    %Assemble composition matrix
C0  = C;                    %Store initial
%------------------------------------------------------------------------
% Variable Initialization
t   = 0;
T   = Ti;
P   = Pi;
it  = 0;
while T>Tstop
    it = it + 1;
    t  = t + dtdiff;
    T  = interp1(tpath,Tpath,t);
    P  = Pi;
    %----------------------------------------------------------------
    [C3,r2,Dend3] = diffusion_grt_v8_VECT(C,R,T,P,dtdiff,ndim,NBC,nrest);
    C             = C3;
    %----------------------------------------------------------------
    if mod(it,nout)==0 && plot_sim == 1
        figure(1),clf
        subplot(211)
        plot(tpath/Myr,Tpath-273,t/Myr,T-273,'r o','Linewidth',1),grid on
        xlabel('t(My)','Interpreter','latex'),ylabel('T $(^oC)$','Interpreter','latex'),title('(a)')
        subplot(212)
        plot(R*1e6,C3(1,:),'color',[0 0.4470 0.7410],'Linewidth',1),hold on
        plot(R*1e6,C3(2,:),'color',[0.8500 0.3250 0.0980],'Linewidth',1)
        plot(R*1e6,C3(3,:),'color',[0.9290 0.6940 0.1250],'Linewidth',1)
        plot(R*1e6,C3(4,:),'color',[0.4940 0.1840 0.5560],'Linewidth',1)
        plot(R*1e6,C0(1,:),'--','color',[0 0.4470 0.7410],'Linewidth',1)
        plot(R*1e6,C0(2,:),'--','color',[0.8500 0.3250 0.0980],'Linewidth',1)
        plot(R*1e6,C0(3,:),'--','color',[0.9290 0.6940 0.1250],'Linewidth',1)
        plot(R*1e6,C0(4,:),'--','color',[0.4940 0.1840 0.5560],'Linewidth',1)
        hold off,grid on
        xlabel('x $(\mu m)$','Interpreter','latex'),ylabel('$X_i$ (mol fraction)','Interpreter','latex'),title('(b)')
        legend([{'Alm'};{'Prp'};{'Sps'};{'Grs'}],'FontSize',12,'location','NorthEastOutside')
        axis([min(R(:))*1e6 max(R(:))*1e6 min(C3(:)) max(C3(:))+0.05])
        legend boxoff
        drawnow
    end
end
if plot_fin==1
    figure(1),clf
    plot(R*1e6,C3(1,:),'color',[0 0.4470 0.7410],'Linewidth',1),hold on
    plot(R*1e6,C3(2,:),'color',[0.8500 0.3250 0.0980],'Linewidth',1)
    plot(R*1e6,C3(3,:),'color',[0.9290 0.6940 0.1250],'Linewidth',1)
    plot(R*1e6,C3(4,:),'color',[0.4940 0.1840 0.5560],'Linewidth',1)
    plot(R*1e6,C0(1,:),'--','color',[0 0.4470 0.7410],'Linewidth',1)
    plot(R*1e6,C0(2,:),'--','color',[0.8500 0.3250 0.0980],'Linewidth',1)
    plot(R*1e6,C0(3,:),'--','color',[0.9290 0.6940 0.1250],'Linewidth',1)
    plot(R*1e6,C0(4,:),'--','color',[0.4940 0.1840 0.5560],'Linewidth',1)
    hold off,grid on
    xlabel('x $(\mu m)$','Interpreter','latex'),ylabel('$X_i$ (mol fraction)','Interpreter','latex')
    title(['Cooling rate: ',num2str(CR*Myr),' K/Myr - T: ',num2str(Ti-273),' $(^oC)$'],'Interpreter','latex')
    legend([{'Alm'};{'Prp'};{'Sps'};{'Grs'}],'FontSize',12,'location','NorthEastOutside')
    axis([min(R(:))*1e6 max(R(:))*1e6 min(C3(:)) max(C3(:))+0.05])
    legend boxoff
end