function [C2,r,D_endm] = diffusion_grt_v8_VECT(C,R,Tinput,Pinput,dtdiff,ndim,NBC,nrest)
%diffusion_grt_v8_VECT.m  is a function to perform multicomponent diffusion
%in garnet
%
%This program comes with no warranty and it is intended for educational/
%research purposes. For further details please see documentation at:
%GDIFF: a Finite Difference code for the calculation of multicomponent diffusion in garnet 
%doi: 10.5281/zenodo.7805990
%
%Evangelos Moulas, 16 May 2023, JGU Mainz
%------------------------------------------------------------------------
Ma        = 1e6*365.25*24*60*60;%seconds per Ma
nit       = 50;                 %Maximum Non-linear Iterations
echo_iter = 0;                  %Echo the number of Iterations
% INPUTS ===============================================================
Tinput =          Tinput;       % T in K
Pinput =      Pinput/1e4;       % from bar to GPa
%-----------------------------------------------------------------------
SUMC   = repmat(sum(C,1),4,1);  %Renormalize endmembers
C      = C./SUMC;               %-//-
%=======================================================================
Xa     =     C(1,:)'; %Renormalized to 1- Fe
Xb     =     C(2,:)'; %Renormalized to 1- Mg
Xc     =     C(3,:)'; %Renormalized to 1- Mn
Xd     = 1-Xa-Xb-Xc;
%Preprocess------------------------------------------------------------
nx     =                length(Xa); %Resolution
Rx     =             max(R)-min(R); %Calculate radial distance
dx     =                 Rx/(nx-1); %calculate dr for radial coordinates
x      =            R(1):dx:R(end); %spatial coordinate (radial)
xc     = 0.5*(x(2:end)+x(1:end-1)); %Average at midpoints
xl     = xc(1:end-1);     xr    = xc(2:end);
xL     = xl(:).^(ndim-1); xR    = xr(:).^(ndim-1); xC = [x(2:end-1).^(ndim-1)]';
dt     = dtdiff/nrest;
S      = dt/dx/dx;
%Create Temporary arrays-----------------------------------------------
Xat    = Xa;
Xbt    = Xb;
Xct    = Xc;
Xdt    = Xd;
%Calculation of Diffusion coefficients --------------------------------
n    =                          4; % 4 endmembers
I    =                     eye(3); % Unity matrix - Kronecker delta
T    =                     Tinput; % Temperature in K
P    =                 Pinput*1e9; % Pressure in Pa
Rgas =                   8.314e-3; % Gas Constant kJ/K/mol
%Diffusion Data========================================================
%D0r= [6.4*1e-4,1.1*1e-3,5.1*1e-4]*1e-4;        %in m2/s
lnD0r = [-16.5644  -16.0228  -16.7914];
Q0r   = [65.824     67.997      60.569]*4.184;  %in kjoule/mol
DV0   = [5.60      5.30          6.00]*1e-6;     %In m3
%Spatial diffusion coefficients ------------------------------------------
%Initialize Diffusion matrix components
D11 = zeros(1,nx-1);
D12 = zeros(1,nx-1);
D13 = zeros(1,nx-1);
D21 = zeros(1,nx-1);
D22 = zeros(1,nx-1);
D23 = zeros(1,nx-1);
D31 = zeros(1,nx-1);
D32 = zeros(1,nx-1);
D33 = zeros(1,nx-1);
lnD0= zeros(4,nx-1);
itdiff  =         0;
time    =         0;
Cit_old = zeros(3*nx,1);
TolE    =      1e-5;      %Tolerance of Non-linear iterations
%========================================================================
while time<dtdiff
    itdiff = itdiff +1;
    time   = time  +  dt;  
    %Post Process midpoints of composition
    Xao = Xa; Xbo = Xb; Xco = Xc; Xdo = Xd;
    Xat = Xa; Xbt = Xb; Xct = Xc; Xdt = Xd;
    for it_n = 1:nit
        Xac = 0.5*(Xat(1:end-1)+Xat(2:end));
        Xbc = 0.5*(Xbt(1:end-1)+Xbt(2:end));
        Xcc = 0.5*(Xct(1:end-1)+Xct(2:end));
        Xdc = 0.5*(Xdt(1:end-1)+Xdt(2:end));
        %----------------------------------------------------------------------
        for ix = 1:nx-1 %For all flux points (middle of grid points)
            lnD0([1:3],ix)= lnD0r(1:3)'  -(Q0r([1:3])' + (P-1e5)*DV0([1:3])'.*1e-3)./Rgas./(T) +  1/6*log(1);
            lnD0(4,ix)    = log(exp(lnD0(1,ix))./2); %Half of Iron (Chakraborty & Ganguly 1992, Fig.4)
            D0(:,ix)      = exp(lnD0(:,ix));
            coupl_on = 1; %CAUTION - USE ONLY FOR DEBUG - will deactivate coupling
            D11(ix)  = D0(1,ix)*I(1,1)-coupl_on*((D0(1,ix)*Xac(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(1,ix)-D0(4,ix));
            D12(ix)  = D0(1,ix)*I(1,2)-coupl_on*((D0(1,ix)*Xac(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(2,ix)-D0(4,ix));
            D13(ix)  = D0(1,ix)*I(1,3)-coupl_on*((D0(1,ix)*Xac(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(3,ix)-D0(4,ix));
            D21(ix)  = D0(2,ix)*I(2,1)-coupl_on*((D0(2,ix)*Xbc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(1,ix)-D0(4,ix));
            D22(ix)  = D0(2,ix)*I(2,2)-coupl_on*((D0(2,ix)*Xbc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(2,ix)-D0(4,ix));
            D23(ix)  = D0(2,ix)*I(2,3)-coupl_on*((D0(2,ix)*Xbc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(3,ix)-D0(4,ix));
            D31(ix)  = D0(3,ix)*I(3,1)-coupl_on*((D0(3,ix)*Xcc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(1,ix)-D0(4,ix));
            D32(ix)  = D0(3,ix)*I(3,2)-coupl_on*((D0(3,ix)*Xcc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(2,ix)-D0(4,ix));
            D33(ix)  = D0(3,ix)*I(3,3)-coupl_on*((D0(3,ix)*Xcc(ix))/(D0(:,ix)'*[Xac(ix);Xbc(ix);Xcc(ix);Xdc(ix)]))*(D0(3,ix)-D0(4,ix));
        end
        D_endm  = [D0(1,:)';D0(2,:)';D0(3,:)'];
        %Preprocess for Matrix Assembly -----------------
        D11r = D11(2:end)';     D11l = D11(1:end-1)';
        D12r = D12(2:end)';     D12l = D12(1:end-1)';
        D13r = D13(2:end)';     D13l = D13(1:end-1)';
        D21r = D21(2:end)';     D21l = D21(1:end-1)';
        D22r = D22(2:end)';     D22l = D22(1:end-1)';
        D23r = D23(2:end)';     D23l = D23(1:end-1)';
        D31r = D31(2:end)';     D31l = D31(1:end-1)';
        D32r = D32(2:end)';     D32l = D32(1:end-1)';
        D33r = D33(2:end)';     D33l = D33(1:end-1)';
        %Create Diagonals (Block 11) -------------------
        ld   = -[D11l*S.*xL;       0;                        0];
        md   =  [0;    (1.*xC + D11l*S.*xL + D11r*S.*xR);    0];
        ud   = -[0;                      0;         D11r*S.*xR];
        L11B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 12) -------------------
        ld   = -[D12l*S.*xL;           0;              0];
        md   =  [0;       ( D12l*S.*xL + D12r*S.*xR);  0];
        ud   = -[0;             0;            D12r*S.*xR];
        L12B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 13) -------------------
        ld   = -[D13l*S.*xL;        0;                 0];
        md   =  [0;       ( D13l*S.*xL+D13r*S.*xR);    0];
        ud   = -[0;             0;            D13r*S.*xR];
        L13B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 21) -------------------
        ld   = -[D21l*S.*xL;        0;                 0];
        md   =  [0;       ( D21l*S.*xL+D21r*S.*xR);    0];
        ud   = -[0;             0;            D21r*S.*xR];
        L21B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 22)
        ld   = -[D22l*S.*xL;               0;                0];
        md   =  [0;    (1.*xC + D22l*S.*xL + D22r*S.*xR);    0];
        ud   = -[0;                          0;     D22r*S.*xR];
        L22B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 23) -------------------
        ld   = -[D23l*S.*xL;           0;                0];
        md   =  [0;       ( D23l*S.*xL + D23r*S.*xR);    0];
        ud   = -[0;                    0;       D23r*S.*xR];
        L23B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 31) -------------------
        ld   = -[D31l*S.*xL;           0;                0];
        md   =  [0;       ( D31l*S.*xL + D31r*S.*xR);    0];
        ud   = -[0;             0;              D31r*S.*xR];
        L31B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 32) -------------------
        ld   = -[D32l*S.*xL;        0;                 0];
        md   =  [0;       ( D32l*S.*xL+D32r*S.*xR);    0];
        ud   = -[0;             0;            D32r*S.*xR];
        L32B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Create Diagonals (Block 33)
        ld   = -[D33l*S.*xL;              0;                0];
        md   =  [0;   (1.*xC + D33l*S.*xL + D33r*S.*xR);    0];
        ud   = -[0;                       0;       D33r*S.*xR];
        L33B = spdiags([ld,md,ud],[-1:1],nx,nx);
        %Assemble Matrix
        LHS  = [L11B,L12B,L13B;
                L21B,L22B,L23B;
                L31B,L32B,L33B];
        RHS1 = [xC(:).*Xao(2:end-1)];
        RHS2 = [xC(:).*Xbo(2:end-1)];
        RHS3 = [xC(:).*Xco(2:end-1)];
        RHS  = [[Xao(1);RHS1(:);Xao(end)];
               [Xbo(1);RHS2(:);Xbo(end)];
               [Xco(1);RHS3(:);Xco(end)]];
        %BC -------------------------------------------
        LHS(1,:)      = 0; LHS(1,1)             = 1;
        LHS(nx,:)     = 0; LHS(nx,nx)           = 1;
        LHS(nx+1,:)   = 0; LHS(nx+1,nx+1)       = 1;
        LHS(2*nx,:)   = 0; LHS(2*nx,2*nx)       = 1;
        LHS(2*nx+1,:) = 0; LHS(2*nx+1,2*nx+1)   = 1;
        LHS(3*nx,:)   = 0; LHS(3*nx,3*nx)       = 1;
        %-----------------------------------------------
        if NBC == 1 || ndim ~= 1
            LHS(1,1+1)         = -1; RHS(1)      = 0;
            LHS(nx+1,nx+2)     = -1; RHS(nx+1)   = 0;
            LHS(2*nx+1,2*nx+2) = -1; RHS(2*nx+1) = 0;
        end
        %Solve -----------------------------------------
        Csol          = LHS\RHS;
        Xat           = Csol(1:nx);
        Xbt           = Csol(nx+1:2*nx);
        Xct           = Csol(2*nx+1:3*nx);
        % closure ------------------------------------------------------------
        Xdt      =     1-Xat-Xbt-Xct;
        if max(abs(Csol-Cit_old))<TolE
            if echo_iter == 1
                disp(['Exit iterations: ',num2str(it_n)])
            end
            break;
        else
           Cit_old = Csol;
        end
        
    end
    Xa      = Xat;
    Xb      = Xbt;
    Xc      = Xct;
    Xd      = Xdt;
end;
C2      = [Xat';Xbt';Xct';Xdt'];
r       = x;
return;